
{% extends "admin/base.html" %}

{% block title %}
{% if subtitle %}{{ subtitle }} | {% endif %}
{{ title }} |
    {% if request.user.is_authenticated and PROJECT_NAME and ENV_NAME %}
        {{PROJECT_NAME}} | {{ENV_NAME}} env |
    {% endif %}
{{ site_title|default:_('Django site admin') }}
{% endblock %}


{% block branding %}
<h1 id="site-name"><a href="{% url 'admin:index' %}">{{ site_header|default:_('Django administration') }}</a></h1>
{% if user.is_anonymous %}
  {% include "admin/color_theme_toggle.html" %}
{% endif %}
{% endblock %}

{% block nav-global %}
{% if request.user.is_authenticated %}
    <noscript>
        <span style="
            background: #b21717;  {# red #}
            color: white;
            font-weight: bold;
            padding: 0 0.5em 0.25em 0.5em;
            ">
            JavaScript is disabled!
        </span>
    </noscript>
{% endif %}
{% endblock %}

